﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Diagnostics;
using System.Xml;
using System.Globalization;


namespace BenjaminSchroeter.GeoNames
{
    public static partial class GeoNamesOrgWebservice
    {
        #region FindCities
        /// <summary>
        /// Finds all cities within a country
        /// </summary>
        /// <param name="country">country code or ISO</param>
        /// <param name="langague">language code or ISO. Default is blank</param>
        /// <param name="startsWith"> Default is blank</param>
        /// <param name="maxRow">number of items to return. Default is return all rows</param>
        /// <returns>generic list: name, state and geonameId</returns>
        public static List<Cities> FindCities(string country, string langague, string startsWith , int maxRow)
        {
            if(string.IsNullOrEmpty(langague ))langague="en";
            if(string.IsNullOrEmpty(startsWith ))startsWith = "";
            //if(maxRow==null)maxRow = 0;
            string xml = FindCitiesXML(country, langague, startsWith, maxRow);

            return CreateCitiesList(xml);
        }

        /// <summary>
        /// Find all cities within a country
        /// </summary>
        /// <remarks>To force search results to return only cities use featureClass = P. See web site for complete list.</remarks>
        /// <param name="country">country code or ISO</param>
        /// <param name="langague">language code or ISO. Default is blank</param>
        /// <param name="startsWith"> Default is blank</param>
        /// <param name="maxRow">number of items to return. Default is return all rows</param>
        /// <returns>XML document.  see web site for full list of elements</returns>
        private static string FindCitiesXML(string country, string langague, string startsWith , int maxRow )
        {
             if(string.IsNullOrEmpty(langague ))langague="en";
            if(string.IsNullOrEmpty(startsWith ))startsWith = "";
            //if(maxRow==null)maxRow = 0;
            string rows = "";
            if (maxRow > 0)
            {
                rows = maxRow.ToString();
            }

            NumberFormatInfo format = new NumberFormatInfo { NumberDecimalSeparator = "." };
            string url = String.Format(format, "http://ws.geonames.org/search?country={0}&lang={1}&name_startsWith={2}&maxRows={3}&style={4}&featureClass={5}", country, langague, startsWith, rows, GeoNamesDataStyle.Full, GeoNamesFeatureClass.P);

            Debug.WriteLine("Requesting " + url);

            string xml = CreateWebClient().DownloadString(url);
            GeonameException.ThrowOnError(xml);
            return xml;
        }

        private static List<Cities> CreateCitiesList(string xml)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(xml);
            if (doc.DocumentElement == null)
                return new List<Cities>();

            XmlNodeList root = doc.GetElementsByTagName("geonames");
            if (root.Count == 0)
                return new List<Cities>();
            XmlElement geonames = (XmlElement)root[0];

            List<Cities> results = new List<Cities>();
            foreach (XmlElement element in geonames.GetElementsByTagName("geoname"))
            {
                Cities n = new Cities(element);
                results.Add(n);
            }

            return results;
        }
        #endregion

    }


    [DebuggerDisplay("{Title} ({Language})")]
    public class Cities : GeonameDataBase
    {

        /*
  <toponymName>Haifa</toponymName> 
  <name>Haifa</name> 
  <lat>32.81556</lat> 
  <lng>34.98917</lng> 
  <geonameId>294801</geonameId> 
  <countryCode>IL</countryCode> 
  <countryName>Israel
     */

        #region properties
        [Category("Names")]
        public string Name { get; private set; }
        [Category("Names")]
        public string State { get; private set; }
        [Category("Names")]
        public string GeonameId { get; private set; }

        #endregion

        public Cities(string genonameXml)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(genonameXml);

            if (doc.DocumentElement == null)
                return;

            XmlNodeList root = doc.GetElementsByTagName("entry");
            if (root.Count == 0)
                return;

            xml = (XmlElement)root[0];

            setProperties();
        }

        public Cities(XmlElement geoname)
        {
            xml = geoname;
            setProperties();
        }

        private void setProperties()
        {
            this.Name = GetElement(xml, "name");
            this.State = GetElement(xml, "adminCode1"); // this is the element from the returned XML from the web service 
            this.GeonameId = GetElement(xml, "geonameId");
        }
    }
}
